==> pyimpspec-3.1.3.tar.gz: pyimpspec-3.1.3/src/pyimpspec/data/formats/p00.py <==
# pyimpspec is licensed under the GPLv3 or later (https://www.gnu.org/licenses/gpl-3.0.html).
# Copyright 2022 pyimpspec developers
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# The licenses of pyimpspec's dependencies and/or sources of portions of code are included in
# the LICENSES folder.

from os.path import exists
from typing import (
    IO,
    List,
)
from pyimpspec.data.data_set import (
    DataFrame,
    DataSet,
    dataframe_to_dataset,
)


def parse_p00(path: str) -> DataSet:
    """
    Parse a .P00 file containing an impedance spectrum.

    Parameters
    ----------
    path: str
        The path to the file to process.

    Returns
    -------
    DataSet
    """
    assert type(path) is str and exists(path)
    fp: IO
    with open(path, "r", encoding="latin1") as fp:
        lines: List[str] = list(
            filter(lambda _: _ != "", map(str.lower, map(str.strip, fp.readlines())))
        )
    num_points: int = 0
    while lines:
        line: str = lines.pop(0)
        if line.startswith("f/hz"):
            num_points = int(lines.pop(0))
            break
    freq: List[float] = []
    real: List[float] = []
    imag: List[float] = []
    while lines:
        columns: List[str] = lines.pop(0).split("\t")
        assert len(columns) == 6, (
            len(columns),
            columns,
        )
        freq.append(float(columns[0]))
        real.append(float(columns[1]))
        imag.append(-float(columns[2]))
    assert len(freq) == len(real) == len(imag) == num_points > 0, len(freq)
    return dataframe_to_dataset(
        DataFrame.from_dict(
            {
                "frequency": freq,
                "real": real,
                "imaginary": imag,
            }
        ),
        path,
        "",
    )

==> pyimpspec-5.0.1.tar.gz: pyimpspec-5.0.1/src/pyimpspec/data/formats/p00.py <==
# pyimpspec is licensed under the GPLv3 or later (https://www.gnu.org/licenses/gpl-3.0.html).
# Copyright 2024 pyimpspec developers
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# The licenses of pyimpspec's dependencies and/or sources of portions of code are included in
# the LICENSES folder.

from pyimpspec.data.data_set import (
    DataSet,
    dataframe_to_data_sets,
)
from pyimpspec.exceptions import UnsupportedFileFormat
from pyimpspec.typing.helpers import (
    IO,
    List,
    Path,
    Union,
)
from .helpers import _validate_path


def parse_p00(path: Union[str, Path]) -> List[DataSet]:
    """
    Parse a .P00 file containing an impedance spectrum.

    Parameters
    ----------
    path: Union[str, pathlib.Path]
        The path to the file to process.

    Returns
    -------
    List[DataSet]
    """
    from pandas import DataFrame

    _validate_path(path)

    fp: IO
    with open(path, "r", encoding="latin1") as fp:
        lines: List[str] = list(
            filter(lambda _: _ != "", map(str.lower, map(str.strip, fp.readlines())))
        )

    num_points: int = 0
    while lines:
        line: str = lines.pop(0)
        if line.startswith("f/hz"):
            num_points = int(lines.pop(0))
            break

    freq: List[float] = []
    real: List[float] = []
    imag: List[float] = []

    while lines:
        columns: List[str] = lines.pop(0).split("\t")

        if len(columns) != 6:
            raise UnsupportedFileFormat(f"Expected to parse 6 columns instead of {len(columns)} columns")

        freq.append(float(columns[0]))
        real.append(float(columns[1]))
        imag.append(-float(columns[2]))

    if not (len(freq) == len(real) == len(imag) > 0):
        raise ValueError(
            "Expected at least one set of frequency, real, and imaginary values"
        )
    elif len(freq) != num_points:
        raise ValueError(f"Expected to parse {num_points} data points instead of {len(freq)}")

    return dataframe_to_data_sets(
        DataFrame.from_dict(
            {
                "frequency": freq,
                "real": real,
                "imaginary": imag,
            }
        ),
        path=path,
    )
